@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  * vim: sw=4 ts=4 et :
  */
 /* This Source Code Form is subject to the terms of the Mozilla Public
@@ -121,11 +121,13 @@ struct Trigger
 
     Trigger(Action action, int32_t msg) :
         mAction(action),
-        mMsg(msg)
-    {}
+        mMessage(msg)
+    {
+      MOZ_ASSERT(0 <= msg && msg < INT32_MAX);
+    }
 
-    Action mAction;
-    int32_t mMsg;
+    uint32_t mAction : 1;
+    uint32_t mMessage : 31;
 };
 
 class ProtocolCloneContext
@@ -183,10 +185,6 @@ public:
     virtual ProcessId OtherPid() const = 0;
     virtual MessageChannel* GetIPCChannel() = 0;
 
-    // The implementation of function is generated by code generator.
-    virtual void CloneManagees(ListenerT* aSource,
-                               ProtocolCloneContext* aCtx) = 0;
-
     Maybe<ListenerT*> ReadActor(const IPC::Message* aMessage, PickleIterator* aIter, bool aNullable,
                                 const char* aActorDescription, int32_t aProtocolTypeId);
 };
@@ -218,23 +216,14 @@ class Endpoint;
  * IToplevelProtocol tracks all top-level protocol actors created from
  * this protocol actor.
  */
-class IToplevelProtocol : private LinkedListElement<IToplevelProtocol>
+class IToplevelProtocol
 {
-    friend class LinkedList<IToplevelProtocol>;
-    friend class LinkedListElement<IToplevelProtocol>;
-
     template<class PFooSide> friend class Endpoint;
 
 protected:
     explicit IToplevelProtocol(ProtocolId aProtoId);
     ~IToplevelProtocol();
 
-    /**
-     * Add an actor to the list of actors that have been opened by this
-     * protocol.
-     */
-    void AddOpenedActor(IToplevelProtocol* aActor);
-
 public:
     void SetTransport(UniquePtr<Transport> aTrans)
     {
@@ -245,33 +234,9 @@ public:
 
     ProtocolId GetProtocolId() const { return mProtocolId; }
 
-    void GetOpenedActors(nsTArray<IToplevelProtocol*>& aActors);
-
     virtual MessageChannel* GetIPCChannel() = 0;
 
-    // This Unsafe version should only be used when all other threads are
-    // frozen, since it performs no locking. It also takes a stack-allocated
-    // array and its size (number of elements) rather than an nsTArray. The Nuwa
-    // code that calls this function is not allowed to allocate memory.
-    size_t GetOpenedActorsUnsafe(IToplevelProtocol** aActors, size_t aActorsMax);
-
-    virtual IToplevelProtocol*
-    CloneToplevel(const InfallibleTArray<ProtocolFdMapping>& aFds,
-                  base::ProcessHandle aPeerProcess,
-                  ProtocolCloneContext* aCtx);
-
-    void CloneOpenedToplevels(IToplevelProtocol* aTemplate,
-                              const InfallibleTArray<ProtocolFdMapping>& aFds,
-                              base::ProcessHandle aPeerProcess,
-                              ProtocolCloneContext* aCtx);
-
 private:
-    void AddOpenedActorLocked(IToplevelProtocol* aActor);
-    void GetOpenedActorsLocked(nsTArray<IToplevelProtocol*>& aActors);
-
-    LinkedList<IToplevelProtocol> mOpenActors; // All protocol actors opened by this.
-    IToplevelProtocol* mOpener;
-
     ProtocolId mProtocolId;
     UniquePtr<Transport> mTrans;
 };
@@ -525,13 +490,8 @@ public:
     }
 
     // This method binds aActor to this endpoint. After this call, the actor can
-    // be used to send and receive messages. The endpoint becomes invalid. The
-    // |aProcessActor| parameter is used to associate protocols with content
-    // processes. In practice, this parameter should always be a ContentParent
-    // or ContentChild, depending on which process you are in. It is used to
-    // find all the channels that need to be "frozen" or "revived" when creating
-    // or cloning the Nuwa process.
-    bool Bind(PFooSide* aActor, IToplevelProtocol* aProcessActor)
+    // be used to send and receive messages. The endpoint becomes invalid.
+    bool Bind(PFooSide* aActor)
     {
         MOZ_RELEASE_ASSERT(mValid);
         MOZ_RELEASE_ASSERT(mMyPid == base::GetCurrentProcId());
@@ -546,9 +506,6 @@ public:
         }
         mValid = false;
         aActor->SetTransport(Move(t));
-        if (aProcessActor) {
-            aProcessActor->AddOpenedActor(aActor);
-        }
         return true;
     }
 
@@ -566,6 +523,13 @@ private:
     ProtocolId mProtocolId;
 };
 
+#if defined(MOZ_CRASHREPORTER) && defined(XP_MACOSX)
+void AnnotateCrashReportWithErrno(const char* tag, int error);
+#else
+static inline void AnnotateCrashReportWithErrno(const char* tag, int error)
+{}
+#endif
+
 // This function is used internally to create a pair of Endpoints. See the
 // comment above Endpoint for a description of how it might be used.
 template<class PFooParent, class PFooChild>
@@ -584,6 +548,7 @@ CreateEndpoints(const PrivateIPDLInterface& aPrivate,
   TransportDescriptor parentTransport, childTransport;
   nsresult rv;
   if (NS_FAILED(rv = CreateTransport(aParentDestPid, &parentTransport, &childTransport))) {
+    AnnotateCrashReportWithErrno("IpcCreateEndpointsNsresult", int(rv));
     return rv;
   }
 
@@ -596,10 +561,31 @@ CreateEndpoints(const PrivateIPDLInterface& aPrivate,
   return NS_OK;
 }
 
+void
+TableToArray(const nsTHashtable<nsPtrHashKey<void>>& aTable,
+             nsTArray<void*>& aArray);
+
 } // namespace ipc
 
 template<typename Protocol>
-using ManagedContainer = nsTHashtable<nsPtrHashKey<Protocol>>;
+class ManagedContainer : public nsTHashtable<nsPtrHashKey<Protocol>>
+{
+  typedef nsTHashtable<nsPtrHashKey<Protocol>> BaseClass;
+
+public:
+  // Having the core logic work on void pointers, rather than typed pointers,
+  // means that we can have one instance of this code out-of-line, rather
+  // than several hundred instances of this code out-of-lined.  (Those
+  // repeated instances don't necessarily get folded together by the linker
+  // because they contain member offsets and such that differ between the
+  // functions.)  We do have to pay for it with some eye-bleedingly bad casts,
+  // though.
+  void ToArray(nsTArray<Protocol*>& aArray) const {
+    ::mozilla::ipc::TableToArray(*reinterpret_cast<const nsTHashtable<nsPtrHashKey<void>>*>
+                                 (static_cast<const BaseClass*>(this)),
+                                 reinterpret_cast<nsTArray<void*>&>(aArray));
+  }
+};
 
 template<typename Protocol>
 Protocol*
