@@ -30,7 +30,7 @@
 #include "ssl.h"
 #include "sslproto.h"
 #include "plhash.h"
-#include "mozilla/Snprintf.h"
+#include "mozilla/Sprintf.h"
 
 using namespace mozilla;
 using namespace mozilla::psm;
@@ -527,7 +527,7 @@ bool AdjustWebSocketHost(relayBuffer& buffer, connection_info_t *ci)
   char newhost[40];
   PR_NetAddrToString(&inet_addr, newhost, sizeof(newhost));
   assert(strlen(newhost) < sizeof(newhost) - 7);
-  snprintf_literal(newhost, "%s:%d", newhost, PR_ntohs(inet_addr.inet.port));
+  SprintfLiteral(newhost, "%s:%d", newhost, PR_ntohs(inet_addr.inet.port));
 
   int diff = strlen(newhost) - (endhost-host);
   if (diff > 0)
@@ -1557,10 +1557,13 @@ int main(int argc, char** argv)
   // Initialize NSS
   if (NSS_Init(nssconfigdir.c_str()) != SECSuccess) {
     int32_t errorlen = PR_GetErrorTextLength();
-    char* err = new char[errorlen+1];
-    PR_GetErrorText(err);
-    LOG_ERROR(("Failed to init NSS: %s", err));
-    delete[] err;
+    if (errorlen) {
+      auto err = mozilla::MakeUnique<char[]>(errorlen + 1);
+      PR_GetErrorText(err.get());
+      LOG_ERROR(("Failed to init NSS: %s", err.get()));
+    } else {
+      LOG_ERROR(("Failed to init NSS: Cannot get error from NSPR."));
+    }
     PR_ShutdownThreadPool(threads);
     PR_DestroyCondVar(shutdown_condvar);
     PR_DestroyLock(shutdown_lock);
