@@ -96,10 +96,8 @@ var snapshotFormatters = {
       $("crashes-noConfig").classList.remove("no-copy");
       return;
     }
-    else {
-      $("crashes-allReports").style.display = "block";
-      $("crashes-allReports").classList.remove("no-copy");
-    }
+    $("crashes-allReports").style.display = "block";
+    $("crashes-allReports").classList.remove("no-copy");
 
     if (data.pending > 0) {
       $("crashes-allReportsWithPending").textContent =
@@ -162,7 +160,7 @@ var snapshotFormatters = {
         $.new("td", experiment.active),
         $.new("td", experiment.endDate),
         $.new("td", [
-          $.new("a", experiment.detailURL, null, {href : experiment.detailURL,})
+          $.new("a", experiment.detailURL, null, {href : experiment.detailURL, })
         ]),
         $.new("td", experiment.branch),
       ]);
@@ -302,10 +300,11 @@ var snapshotFormatters = {
           let assembled = assembleFromGraphicsFailure(i, data);
           combined.push(assembled);
         }
-        combined.sort(function(a,b) {
+        combined.sort(function(a, b) {
             if (a.index < b.index) return -1;
             if (a.index > b.index) return 1;
-            return 0;});
+            return 0;
+        });
         $.append($("graphics-failures-tbody"),
                  combined.map(function(val) {
                    return $.new("tr", [$.new("th", val.header, "column"),
@@ -572,14 +571,6 @@ var snapshotFormatters = {
           data[key] === data["hasUserNamespaces"]) {
         continue;
       }
-      // Hard code content sandbox label on Fx50 to prevent noise in l10n tools.
-      if (key === "contentSandboxLevel") {
-        tbody.appendChild($.new("tr", [
-          $.new("th", "Content Process Sandbox Level", "column"),
-          $.new("td", data[key])
-        ]));
-        continue;
-      }
       tbody.appendChild($.new("tr", [
         $.new("th", strings.GetStringFromName(key), "column"),
         $.new("td", data[key])
@@ -958,21 +949,21 @@ function safeModeRestart() {
 /**
  * Set up event listeners for buttons.
  */
-function setupEventListeners(){
+function setupEventListeners() {
   $("show-update-history-button").addEventListener("click", function (event) {
     var prompter = Cc["@mozilla.org/updates/update-prompt;1"].createInstance(Ci.nsIUpdatePrompt);
       prompter.showUpdateHistory(window);
   });
-  $("reset-box-button").addEventListener("click", function (event){
+  $("reset-box-button").addEventListener("click", function (event) {
     ResetProfile.openConfirmationDialog(window);
   });
-  $("copy-raw-data-to-clipboard").addEventListener("click", function (event){
+  $("copy-raw-data-to-clipboard").addEventListener("click", function (event) {
     copyRawDataToClipboard(this);
   });
-  $("copy-to-clipboard").addEventListener("click", function (event){
+  $("copy-to-clipboard").addEventListener("click", function (event) {
     copyContentsToClipboard();
   });
-  $("profile-dir-button").addEventListener("click", function (event){
+  $("profile-dir-button").addEventListener("click", function (event) {
     openProfileDirectory();
   });
   $("restart-in-safe-mode-button").addEventListener("click", function (event) {
@@ -983,7 +974,7 @@ function setupEventListeners(){
       safeModeRestart();
     }
   });
-  $("verify-place-integrity-button").addEventListener("click", function (event){
+  $("verify-place-integrity-button").addEventListener("click", function (event) {
     PlacesDBUtils.checkAndFixDatabase(function(aLog) {
       let msg = aLog.join("\n");
       $("verify-place-result").style.display = "block";
